
//Create and init variables.
var gravity=0.01;
var droids = [];
var minX=0.02, maxX=0.90;
var minY=0.02, maxY=0.90;
var imgDroid, toggle=false;
var fps = 60;
var time = new Date().getTime();

//Define a droid data object to
//store info about each droid.
function Droid( img )
{
    this.img = img;
    this.x = 0; this.y = 0;
    this.speedX=0; this.speedY=0;
    this.angle=0; this.spin=0;
}

//Called when application is started.
function OnStart()
{
	//Lock screen orientation to Portrait.
    app.SetOrientation( "Portrait" );
    
    //Stop screen turning off.
    app.PreventScreenLock( true );
    
    //Create the main layout with a background image.
    lay = app.CreateLayout( "Linear", "FillXY" ); 
    lay.SetBackground( "/Sys/Img/BlackBack.png" );
    
    //Create an blank image to act as our drawing 'canvas'.
    //(For performance reasons, we limit the internal bitmap to 
    //480x800 and set screen updating to manual mode).
    canvas = app.CreateImage( null, 1.0, 1.0, "fix", 480, 800 );
	canvas.SetAutoUpdate( false );
	canvas.SetTextSize( 10 );
    canvas.SetPaintColor( "#ffff0000" );
	canvas.SetOnTouchDown( canvas_OnTouchDown );
	lay.AddChild( canvas );
	
    //Add layout to app. 
    app.AddLayout( lay );
    
    //Create droid image in memory (not added to layout).
	imgDroid = app.CreateImage( "/Sys/Img/Hello.png", 0.15 );

    //Add 1 droid to the droids array.
    AddDroids( 1 );
    
    //Call the DrawFrame function in 100ms.
    setTimeout( DrawFrame, 100 );
    
    //Create two media players.
	player1 = app.CreateMediaPlayer();
	player2 = app.CreateMediaPlayer();

	//Load sound files (can be ogg or mp3).
	player1.SetFile( "/Sys/Snd/Poing.ogg" );
	player2.SetFile( "/Sys/Snd/Poing.ogg" );
	
    //Switch off debugging for max speed.
    //(keep this on during development)
    app.SetDebugEnabled( false );
}

//Handle touching screen.
function canvas_OnTouchDown()
{
    AddDroids( 1 );
}

//Add 'n' droids to droid array.
function AddDroids(n)
{
    for (var i=0; i<n; i++)
    {
        //Create droid object and set speed.
        var droid = new Droid( imgDroid );
        droid.speedX = 0.02 * Math.random();
        droid.speedY = 0.01 * (Math.random()-0.5);
        
        //Add droid object to droid array.
        //stage.addChild(droid);
        droids.push( droid );
    }
}

//Update and redraw all game graphics.
function DrawFrame()
{
    //Clear the canvas.
    canvas.Clear();
    
    //Draw droids.
    DrawDroids();
    
    //Update the canvas.
    canvas.Update();
    
    //Call this func again as soon as pos.
    setTimeout( DrawFrame, 0 );
}

//Draw all the droids in the array.
function DrawDroids()
{
    //Display droid count and frame rate.
    var ntime = new Date().getTime();
    fps = 0.97*fps + 0.03*1000/(ntime-time);  
    time = ntime;
    var txt = droids.length + " droids\nFPS: " + Math.round(fps);
    canvas.DrawText( txt, 0.02, 0.06 );
     
     //Loop through all our droid objects.
     for( var i=0; i<droids.length; i++ )
     {
        //Update droid position.
        var droid = droids[i];
        droid.x += droid.speedX;
        droid.y += droid.speedY;
        droid.speedY += gravity;
        droid.angle += droid.spin;
        
        //Check each droids X position and 
        //reverse direction if hit side wall.
        if( droid.x > maxX ) 
        {
            droid.speedX *= -1;
            droid.x = maxX;
        }
        else if( droid.x < minX )
        {
            droid.speedX *= -1;
            droid.x = minX;
        }
        
        //Check each droids Y range, slow down
        //and reverse direction if hit floor.
        if( droid.y > maxY )
        {
            droid.speedY *= -0.8;
            droid.y = maxY;
            
            //Add random slow down.
            if( Math.random() > 0.5 ) 
                droid.speedY -= 0.03 * Math.random();
                
            //Add random spin.
            if( Math.random() > 0.9 ) 
                droid.spin += 30 * (Math.random()-0.5);
                
            //Play 'boing' sound.
            PlaySound();
        } 
        else if( droid.y < minY )
        {
            droid.speedY = 0;
            droid.y = minY;
        }
        
        //Draw droid to canvas.
        canvas.DrawImage( droid.img, droid.x, droid.y, 
            0.1, 0.06, droid.angle )
    }
}

//Play 'boing' sound.
function PlaySound()
{
    //Alternate between players so that
    //can hear two 'boings' at once.
    var player = player1;
    if( toggle ) player = player2;
    toggle = !toggle;
    
    //Start playing boing sound.
    player.SeekTo( 0 );
    player.Play();
}

